@@ -1,6 +1,6 @@
 from dataclasses import dataclass
 from itertools import repeat
-from typing import Any, Hashable, Iterable, Literal, Mapping, Sequence, Union
+from typing import Any, Hashable, Iterable, Literal, Mapping, Sequence, Union, overload
 
 import numpy as np
 from numpy import typing as npt
@@ -95,6 +95,22 @@ class Grouper:
         np.cumsum(self.counts[:-1], out=indexes[1:])
         return indexes
 
+    def group_indexes(self) -> npt.NDArray[np.int_]:
+        """
+        Calculate the group key indexes.
+
+        Usage:
+        >>> df = DataFrame({"a": [1, 1, 2, 2, 3, 3, 3], "b": [4, 5, 6, 7, 8, 9, 10]})
+        >>> df.groupby("a").group_indexes()
+        array([0, 2, 4])
+        >>> df["a"][df.groupby("a").group_indexes()]
+        array([1, 2, 3])
+        """
+        indexes = np.empty(len(self.counts), dtype=int)
+        indexes[0] = 0
+        np.cumsum(self.counts[:-1], out=indexes[1:])
+        return self.order[indexes]
+
     def __iter__(self):
         """
         Iterate over indexes of each group.
@@ -220,13 +236,30 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
             return 0
         return len(next(iter(self._columns.values())))
 
-    def __getitem__(self, item: Hashable) -> np.ndarray:
+    @overload
+    def __getitem__(self, item: Hashable) -> np.ndarray:  # noqa: D105
+        ...
+
+    @overload
+    def __getitem__(self, item: list[Hashable]) -> "DataFrame":  # noqa: D105
+        ...
+
+    def __getitem__(self, item: Hashable | list[Hashable]) -> Union[np.ndarray, "DataFrame"]:
         """
         Extract the column values by key. Raise `KeyError` if the column doesn't exist.
 
+        If the item is an array, return the DataFrame with the corresponding columns.
+
         :param item: Column key.
         :return: Column values, zero-copy. The user is welcome to mutate the internals.
         """
+        if isinstance(item, list):
+            item = item.copy()
+            columns = self._columns
+            for i in self._index:
+                if i not in item:
+                    item.append(i)
+            return DataFrame({k: columns[k] for k in item}, index=self._index)
         return self._columns[item]
 
     def __setitem__(self, key: Hashable, value: npt.ArrayLike) -> None:
@@ -484,7 +517,7 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
             return df.take(first_found, inplace=inplace)
         return df
 
-    def groupby(self, *by: Hashable) -> Grouper:
+    def groupby(self, *by: Hashable | npt.ArrayLike) -> Grouper:
         """
         Group rows by one or more columns in `by`.
 
@@ -504,7 +537,18 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
         a 3
         b [ 8  9 10]
         """
-        order, values = self._order([self[c] for c in by], "stable")
+        seed = []
+        for c in by:
+            if isinstance(c, (list, np.ndarray)):
+                c = np.asarray(c)
+                if c.shape != (len(self),):
+                    raise ValueError(f"shape mismatch {c.shape} != ({len(self)},)")
+                seed.append(c)
+            else:
+                if not isinstance(c, Hashable):
+                    raise KeyError(f"Invalid column key: {c}")
+                seed.append(self[c])
+        order, values = self._order(seed, "stable")
         _, counts = np.unique(values[order], return_counts=True)
         return Grouper(order, counts)
 
